/*(c) JippySnake7 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.util.TimerTask;
import java.util.LinkedList;

class GameLoop extends TimerTask {
	private JSLevel level;
	private Snake snake;
	private Player player;
	
	private Coord dir;
	private Coord nextDir;
	private boolean movedWithNewDir;
	private GUIUpdater updater;
	
	private boolean paused;
	private boolean drawPaused;
	
	private boolean goDown;
	private boolean appleEaten;
	
	private boolean levelDone;
	
	private Coord prevHeadPos;
	private Coord prevDir;
	
	private LinkedList<Coord> newDirs;
	
	private SoundPlayer soundPlayer;
	
	private int runs;
	
	public GameLoop(JSLevel l, Snake s, GUIUpdater u, Player p) {
		level = l;
		snake = s;
		updater = u;
		player = p;
		dir = Coord.getZeroCoord();
		paused = false;
		prevHeadPos = null;
		nextDir = null;
		movedWithNewDir = true;
		goDown = false;
		prevDir = Coord.getZeroCoord();
		appleEaten = false;
		newDirs = new LinkedList<Coord>();
		soundPlayer = new SoundPlayer(true);
		
		levelDone = false;
		runs = 0;
	}
	
	public void run() {
		Coord crd = snake.getHeadPos();
		if (paused) {
			if (!drawPaused) updater.draw(paused, (level.getType() == 4),crd);
			drawPaused = true;
			return;
		} else drawPaused = false;
		Coord pPos = snake.getPrevBodyPos();
		if (pPos != null) level.setSnake(pPos, true);
		Coord cPos = snake.getBodyPos();
		if (cPos != null) level.setSnake(cPos, false);
		prevHeadPos = snake.getHeadPos();
		
		//set texture of body-part behind head
		snake.setBodyTexture(1999);
		if (level.getNumApples() > 1) {
			if (prevHeadPos.isRightOf(pPos) && dir.equals(new Coord(0,-1))) snake.setBodyTexture(1031);
			if (prevHeadPos.isRightOf(pPos) && dir.equals(new Coord(0,1))) snake.setBodyTexture(1027);
			if (prevHeadPos.isRightOf(pPos) && dir.equals(new Coord(-1,0))) {
				snake.setBodyTexture(1015);
				if (prevHeadPos.getZ() > 0)
					if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),1))
						snake.setBodyTexture(1023);
					else if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),3))
						snake.setBodyTexture(1019);
			}
			if (prevHeadPos.isLeftOf(pPos) && dir.equals(new Coord(0,-1))) snake.setBodyTexture(1025);
			if (prevHeadPos.isLeftOf(pPos) && dir.equals(new Coord(0,1))) snake.setBodyTexture(1029);
			if (prevHeadPos.isLeftOf(pPos) && dir.equals(new Coord(1,0))) {
				snake.setBodyTexture(1013);
				if (prevHeadPos.getZ() > 0)
					if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),1))
						snake.setBodyTexture(1017);
					else if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),3))
						snake.setBodyTexture(1021);
			}
			if (prevHeadPos.isDownOf(pPos) && dir.equals(new Coord(1,0))) snake.setBodyTexture(1028);
			if (prevHeadPos.isDownOf(pPos) && dir.equals(new Coord(-1,0))) snake.setBodyTexture(1024);
			if (prevHeadPos.isDownOf(pPos) && dir.equals(new Coord(0,-1,0))) {
				snake.setBodyTexture(1012);
				if (prevHeadPos.getZ() > 0)
					if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),0))
						snake.setBodyTexture(1016);
					else if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),2))
						snake.setBodyTexture(1020);
			}
			if (prevHeadPos.isTopOf(pPos) && dir.equals(new Coord(1,0))) snake.setBodyTexture(1026);
			if (prevHeadPos.isTopOf(pPos) && dir.equals(new Coord(-1,0))) snake.setBodyTexture(1030);
			if (prevHeadPos.isTopOf(pPos) && dir.equals(new Coord(0,1))) {
				snake.setBodyTexture(1014);
				if (prevHeadPos.getZ() > 0)
					if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),0))
						snake.setBodyTexture(1022);
					else if (level.isRampType(prevHeadPos.add(new Coord(0,0,-1)),2))
						snake.setBodyTexture(1018);
			}
		}
		snake.move(dir);
		pPos = snake.getPrevBodyPos();
		if (pPos != null) level.setSnake(pPos, true);
		
		crd = snake.getHeadPos();
		
		//set Texture of new snakeEnd
		snake.setBodyTexture(1);
		Coord nPos = snake.getNextBodyPos();
		cPos = snake.getBodyPos();
		if (nPos != null) {
			//snake has only one element
			if (cPos.equals(nPos)) nPos = crd;
			
			if (cPos.sub(nPos).equals(new Coord(0,1,0),"xy")) {
				snake.setBodyTexture(1032); //EndUp
				if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),2)) snake.setBodyTexture(1040);
				else if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),0)) snake.setBodyTexture(1036);
			}
			else if (cPos.sub(nPos).equals(new Coord(0,-1,0),"xy")) {
				snake.setBodyTexture(1034); //endDown
				if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),2)) snake.setBodyTexture(1038);
				else if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),0)) snake.setBodyTexture(1042);
			}
			else if (cPos.sub(nPos).equals(new Coord(1,0,0),"xy")) {
				snake.setBodyTexture(1035); //endLeft
				if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),1)) snake.setBodyTexture(1043);
				else if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),3)) snake.setBodyTexture(1039);
			}
			else if (cPos.sub(nPos).equals(new Coord(-1,0,0),"xy")) {
				snake.setBodyTexture(1033); //endRight
				if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),1)) snake.setBodyTexture(1037);
				else if ((cPos.getZ() > 0) && level.isRampType(cPos.add(new Coord(0,0,-1)),3)) snake.setBodyTexture(1041);
			}
		}
		if (goDown) {
			goDown = false;
			if (prevDir.equals(dir)) {
				snake.setHeadPos(crd.add(new Coord(0,0,-1)));
			} else {
				die();
				return;
			}
		}
		crd = snake.getHeadPos();
		cPos = snake.getBodyPos();
		if ( (crd.getX() < 0) ||
		      (crd.getY() < 0) ||
		      (crd.getX() >= Globals.SIZE_X) ||
		      (crd.getY() >= Globals.SIZE_Y) ||
		      level.getSnake(crd) ||
		      ((!goDown) && (crd.getZ() > 0) && (level.getLayer(crd.add(new Coord(0,0,-1))) == 0)) ) {
			die();
			return;
		}
		
		crd = snake.getHeadPos();
		if (level.isRampType(crd,0))
			if (crd.sub(prevHeadPos).equals(new Coord(0,-1,0),"xy")) {
				snake.setHeadPos(crd.add(new Coord(0,0,1)));
			} else {
				die();
				return;
			}
		else if (level.isRampType(crd,1)) {
			if (crd.sub(prevHeadPos).equals(new Coord(1,0,0),"xy")) {
				snake.setHeadPos(crd.add(new Coord(0,0,1)));
			} else {
				die();
				return;
			}
		} else if (level.isRampType(crd,2))
			if (crd.sub(prevHeadPos).equals(new Coord(0,1,0),"xy")) {
				snake.setHeadPos(crd.add(new Coord(0,0,1)));
			} else {
				die();
				return;
			}
		else if (level.isRampType(crd,3))
			if (crd.sub(prevHeadPos).equals(new Coord(-1,0,0),"xy")) {
				snake.setHeadPos(crd.add(new Coord(0,0,1)));
			} else {
				die();
				return;
			}
		else if (!level.isPassable(crd)) {
			die();
			return;
		}
		
		if (crd.getZ() > 0) {
			if (level.isRampType(crd.add(new Coord(0,0,-1)),0) && crd.sub(prevHeadPos).equals(new Coord(0,1,0))) goDown = true;
			else if (level.isRampType(crd.add(new Coord(0,0,-1)),1) && crd.sub(prevHeadPos).equals(new Coord(-1,0,0))) goDown = true;
			else if (level.isRampType(crd.add(new Coord(0,0,-1)),2) && crd.sub(prevHeadPos).equals(new Coord(0,-1,0))) goDown = true;
			else if (level.isRampType(crd.add(new Coord(0,0,-1)),3) && crd.sub(prevHeadPos).equals(new Coord(1,0,0))) goDown = true;
		}
		if (cPos != null) level.setSnake(cPos,true);
		crd = snake.getHeadPos();
		
		if (level.isAppleAt(crd)) {
			soundPlayer.playSound(soundPlayer.appleSound);
			snake.grow(crd);
			crd = snake.getHeadPos();
			if (!level.removeApple(crd)) {
				player.setLevelDone(level.getType(), level.getLevelNum(), 3);
				player.savePlayer();
				die();
				return;
			}
			appleEaten = true;
		}
		prevDir = dir;
		
		//set HeadTexture
		if (dir.equals(new Coord(0,-1))) {
			snake.setHeadTexture(1000); //HeadUp
			if (crd.getZ() > 0) {
				if (level.isRampType(crd.add(new Coord(0,0,-1)),0)) snake.setHeadTexture(1004);
				else if (level.isRampType(crd.add(new Coord(0,0,-1)),2)) snake.setHeadTexture(1008);
			}
		}
		else if (dir.equals(new Coord(1,0))) {
			snake.setHeadTexture(1001); //headRight
			if (crd.getZ() > 0) {
				if (level.isRampType(crd.add(new Coord(0,0,-1)),1)) snake.setHeadTexture(1005);
				else if (level.isRampType(crd.add(new Coord(0,0,-1)),3)) snake.setHeadTexture(1009);
			}
		}
		else if (dir.equals(new Coord(0,1))) {
			snake.setHeadTexture(1002); //headDown
			if (crd.getZ() > 0) {
				if (level.isRampType(crd.add(new Coord(0,0,-1)),2)) snake.setHeadTexture(1006);
				else if (level.isRampType(crd.add(new Coord(0,0,-1)),0)) snake.setHeadTexture(1010);
			}
		}
		else if (dir.equals(new Coord(-1,0))) {
			snake.setHeadTexture(1003); //headLeft
			if (crd.getZ() > 0) {
				if (level.isRampType(crd.add(new Coord(0,0,-1)),3)) snake.setHeadTexture(1007);
				else if (level.isRampType(crd.add(new Coord(0,0,-1)),1)) snake.setHeadTexture(1011);
			}
		}
		if ((level.getType() == 3) && level.timerUsed() && !dir.equals(Coord.getZeroCoord())) level.decTime();
		if (level.remainingTime() < 0) die();
		if (level.getNumApples() == level.getMinApples()) {
			if (!levelDone) soundPlayer.playSound(soundPlayer.levelDoneSound);
			levelDone = true;
			level.setTimerUsed(false);
			if (player.getLevelDone(level.getType(), level.getLevelNum()) < 2) {
				player.unLockLevel(level.getType());
				player.setLevelDone(level.getType(), level.getLevelNum(),2);
				player.savePlayer();
			}
			if (level.getType() == 1) nextLevel();
		}
		if (!newDirs.isEmpty()) {
			soundPlayer.playSound(soundPlayer.turnSound);
			dir = newDirs.remove();
		} else {
			movedWithNewDir = true;
		}
		updater.draw(paused, (level.getType() == 4),crd);
		runs++;
	}
	
	private void nextLevel() {
		snake.clear();
		level.loadNewLevel("res/levels/level"+level.getType()+"_"+Math.min(level.getLevelNum()+1,99)+".js7l");
		levelDone = (player.getLevelDone(level.getType(), level.getLevelNum()) > 1);
		snake.place(level);
		snake.place(level);
		snake.setHeadTexture(1000);
		dir = Coord.getZeroCoord();
		prevHeadPos = null;
		goDown = false;
		updater.setLevel(level);
		updater.resetReachedUltraSnake();
		updater.drawLevel();
		updater.draw(paused, (level.getType() == 4), snake.getHeadPos());
	}
	
	private void die() {
		soundPlayer.playSound(soundPlayer.dieSound);
		player.setLongestSnake(level.getType(), level.getLevelNum(), level.getNumApples());
		player.savePlayer();
		snake.clear();
		level.reInit();
		snake.place(level);
		snake.setHeadTexture(1000);
		dir = Coord.getZeroCoord();
		prevHeadPos = null;
		goDown = false;
		levelDone = false;
		updater.resetReachedUltraSnake();
		//updater.draw(paused, (level.getType() == 4), snake.getHeadPos());
		updater.prepareHud();
	}
	
	public void setDir(Coord newDir) {
		if (movedWithNewDir) {
			if ((newDir.getX() == -1*dir.getX()) && (newDir.getY() == -1*dir.getY())) return;
			soundPlayer.playSound(soundPlayer.turnSound);
			dir = newDir;
			movedWithNewDir = false;	
		} else {
			if (newDirs.isEmpty()) {
				if ((newDir.getX() == -1*dir.getX()) && (newDir.getY() == -1*dir.getY())) return;
				newDirs.addLast(newDir);
			} else {
				Coord lastDir;
				lastDir = newDirs.getLast();
				if ((newDir.getX() == -1*lastDir.getX()) && (newDir.getY() == -1*lastDir.getY())) return;
				if (newDirs.size() > 1) return;
				newDirs.addLast(newDir);
			}
			//nextDir = newDir;
		}
	}
	
	public void setPaused(boolean paused) {
		this.paused = paused;
	}
	public boolean getPaused() {
		return paused;
	}
	
}