/*(c) JippySnake Entwicklungsteam.
 *
 *
 *Dieser Code wird unter der GNU Public Licence vertireben
 *
 *
 */

/* Modified for Shopping-ManaJer */

import java.awt.Graphics;
import java.util.HashMap;
import java.awt.image.BufferedImage;
import java.awt.image.BufferStrategy;
import java.net.URL;
import java.net.URI;
import javax.imageio.ImageIO;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.io.*;

class LERenderer {
	private Graphics canvasGr;
	private BufferStrategy buffStrat;
	private Canvas canvas;
	private HashMap<String, BufferedImage> hashMap;
	private BufferedImage[] buffImg;
	private Graphics[] buffImgGr;
	public String[] texture;
	
	private String gameRoot;
	private int numLayers;
	
	private int[][][] texts;
	private int w;
	public LERenderer(Canvas c, String textFile, int n, int w) {
		//We need a buttom, a top an 2 layers for each levelLayer
		/* Strukture: N means the number of layers
		                              layer
		               TopLayer        N+2
		               BlockLayer N    N+1
		                    .
		                    .
		                    .
		               BlockLayer 0    1
		               ButtomLayer     0
		*/
		numLayers = n;
		this.w = w;
		texts = new int[numLayers][Globals.SIZE_X][Globals.SIZE_Y];
		loadTextures(textFile);
		initRenderer(c);
	}
	
	private void initRenderer(Canvas c) {
		//init Graphics interface to canvas
		canvas = c;
		c.createBufferStrategy(2);
		buffStrat = c.getBufferStrategy();
		canvasGr = buffStrat.getDrawGraphics();
		
		hashMap = new HashMap<String, BufferedImage>();
		
		buffImg = new BufferedImage[numLayers];
		buffImgGr = new Graphics[numLayers];
		
		for (int i = 0; i < numLayers; i++) {
			buffImg[i] = new BufferedImage(500,500,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[i] = buffImg[i].getGraphics();
		}
		canvasGr.setFont(new Font("",Font.PLAIN,10));
		canvasGr.drawString("",0,0);
		gameRoot = Globals.GAMEROOT;
	}
	
	public void setTexture(int layer, int texNr, Coord crd) {
		texts[layer][crd.getX()][crd.getY()] = texNr;
	}
	
	public void renderLayer(int l) {
		buffImg[l] = new BufferedImage(500,500,BufferedImage.TYPE_INT_ARGB);
		buffImgGr[l] = buffImg[l].getGraphics();
		for (int x = 0; x < Globals.SIZE_X; x++)
			for (int y = 0; y < Globals.SIZE_Y; y++)
				drawImage(l, texture[texts[l][x][y]], new Coord(x,y), -5*l);
	}
	
	public int getTexture(int layer, Coord crd) {
		return texts[layer][crd.getX()][crd.getY()];
	}
	
	public BufferedImage loadImage(String nombre) {
		BufferedImage img = (BufferedImage)hashMap.get(nombre);
		URL url=null;
		if (img == null) {
			try {
				url = new URL(gameRoot+nombre);
				img = ImageIO.read(url);
				hashMap.put(nombre,img);
			} catch (Exception e) {
				System.out.println("Error: "+e.getMessage());
			}
		}
		return img;
	}
	
	public synchronized void drawScreen() {
		for (BufferedImage b : buffImg)
			canvasGr.drawImage(b,0,0,canvas);
		try {
			buffStrat.show();
		} catch (Exception e) {
		}
	}
	
	public void clear(Color c) {
		canvasGr.setColor(c);
		canvasGr.fillRect(0,0,500,500);
	}
	public void drawImage(int l, String txt, Coord pos, int offset) {
		BufferedImage bicho = loadImage(txt);
		try {
			buffImgGr[l].drawImage(bicho, pos.getX()*w+offset, pos.getY()*w+offset, canvas);
		} catch (IndexOutOfBoundsException e) {
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	public String getTexture(int i) {
		if (i > 999) return "end";
		return texture[i];
	}
	private void loadTextures(String file) {
		URI texFile;
		FileReader fileReader;
		String line;
		String[] parts;
		try {
			texFile = new URI(new URL(Globals.GAMEROOT+file).toString());
			fileReader = new FileReader(new File(texFile));
			BufferedReader buffReader = new BufferedReader(fileReader);
			texture = new String[2000];
			for (int i = 0; i < 2000; i++) texture[i] = "";
			line = buffReader.readLine();
			while (!line.equals("@@@")) {
				parts = line.split("::");
				if (parts[0].equals("include")) {
					loadTextures(parts[1]);
				} else {
					texture[Integer.parseInt(parts[0])] = "res/"+parts[1];
				}
				line = buffReader.readLine();
			}
		} catch (FileNotFoundException e) {
			System.out.println("Texture-file not found:"+file);
			System.exit(0);
		} catch (Exception e) {
			System.out.println("Error while reading Textures.");
			e.printStackTrace();
			System.exit(0);
		}
	}
}