/*(c) JippySnake Entwicklungsteam.
 *
 *
 *Dieser Code wird unter der GNU Public Licence vertireben
 *
 *
 */

/* Modified for Shopping-ManaJer */

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferStrategy;
import java.util.HashMap;
import java.net.URL;
import java.net.URI;
import javax.imageio.ImageIO;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.io.*;
import java.awt.RenderingHints;
import java.awt.RenderingHints.Key;

class Renderer {
	private Graphics2D canvasGr;
	private BufferedImage canvasImg;
	private BufferStrategy buffStrat;
	private Canvas canvas;
	private HashMap<RenderingHints.Key, Object> rendHints;
	private BufferedImage[] buffImg;
	private Graphics2D[] buffImgGr;
	public BufferedImage[] texture;
	
	private String gameRoot;
	private int numLayers;
	private int textSize;
	private int offSize;
	
	private int picSizeX;
	private int picSizeY;
	
	public Renderer(Canvas c, String textFile) {
		buffImg = new BufferedImage[12];
		buffImgGr = new Graphics2D[12];
		this.numLayers = 12;
		rendHints = new HashMap<RenderingHints.Key, Object>();
		rendHints.put(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
		gameRoot = Globals.GAMEROOT;
		textSize = 5;
		loadTextures(textFile);
		initRenderer(c);
		offSize = textSize/5;
	}
	
	private void initRenderer(Canvas c) {
		//init Graphics interface to canvas
		canvas = c;
		picSizeX = canvas.getWidth();
		picSizeY = canvas.getHeight();
		
		canvasImg = new BufferedImage(picSizeX, picSizeY,BufferedImage.TYPE_INT_ARGB);
		canvasGr = canvasImg.createGraphics();
		canvasGr.addRenderingHints(rendHints);
		
		//init offscreen layers
		for (int i = 0; i < buffImg.length; i++) {
			buffImg[i] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[i] = buffImg[i].createGraphics();
		}
		canvasGr.setFont(new Font("",Font.PLAIN,10));
		canvasGr.drawString("",0,0);
		
	}
	
	public void setLayers(int l) {
		numLayers = l;
	}
	
	public void drawTexture(int layer, int texNr, int posX, int posY) {
		drawImage(layer, texNr, posX, posY);
	}
	
	public void drawTextureOnScreen(int texNr, int x, int y) {
		drawImageOnScreen(texNr, x, y);
	}
	
	public void clearScreen(Color col, int x, int y, int w, int h) {
		canvasGr.setColor(col);
		canvasGr.fillRect(x,y,w,h);
	}
	
	public void drawRect(int layer, Color col, int x, int y, int w, int h) {
		buffImgGr[layer].setColor(col);
		buffImgGr[layer].fillRect(x,y,w,h);
	}
	
	public void reNewLayer(int layer) {
		buffImg[layer] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
		buffImgGr[layer] = buffImg[layer].createGraphics();
	}
	
	public void reNewAllLayers() {
		for (int layer = 0; layer < buffImg.length; layer++) {
			buffImg[layer] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[layer] = buffImg[layer].createGraphics();
		}
	}
	
	public synchronized void drawScreen() {
		preRender();
		flushScreen();
	}
	
	public void flushScreen() {
		try {
			Graphics g = canvas.getGraphics();
			g.drawImage(canvasImg,0,0,canvas);
		} catch (Exception e) {
		}
	}
	
	public void preRender() {
		BufferedImage b;
		for (int i = 0; i < numLayers; i++) {
			b = buffImg[i];
			canvasGr.drawImage(b, new AffineTransform(), canvas);
		}
	}
	
	public void drawImageOnScreen(int txt, int x, int y) {
		try {
			AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, x, y);
			canvasGr.drawImage(texture[txt], a, canvas);
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	public void drawImage(int layer, int txt, int x, int y) {
		AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, x*1.0, y*1.0);
		try {
			buffImgGr[layer].drawImage(texture[txt], a, canvas);
		} catch (IndexOutOfBoundsException e) {
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	private void loadTextures(String file) {
		URI texFile;
		URL url;
		BufferedImage img;
		FileReader fileReader;
		String line;
		String[] parts;
		try {
			texFile = new URI(new URL(Globals.GAMEROOT+file).toString());
			fileReader = new FileReader(new File(texFile));
			BufferedReader buffReader = new BufferedReader(fileReader);
			texture = new BufferedImage[2000];
			line = buffReader.readLine();
			while (!line.equals("@@@")) {
				parts = line.split("::");
				if (parts[0].equals("include"))
					loadTextures(parts[1]);
				else if (parts[0].equals("size"))
					textSize = Integer.parseInt(parts[1]);
				else {
					url = new URL(gameRoot+"res/"+parts[1]);
					img = ImageIO.read(url);
					texture[Integer.parseInt(parts[0])] = img;
				}
				line = buffReader.readLine();
			}
		} catch (FileNotFoundException e) {
			System.out.println("Texture-file not found:"+file);
			System.exit(0);
		} catch (Exception e) {
			System.out.println("Error while reading Textures.");
			e.printStackTrace();
			System.out.println(e.getMessage());
			System.exit(0);
		}
	}
}